/**
  *  \file server/session.h
  *  \brief Router: Session Implementation
  */
#ifndef C2SERVER_SERVER_SESSION_H
#define C2SERVER_SERVER_SESSION_H

#include <vector>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include "cpluslib/types.h"
#include "cpluslib/string.h"
#include "server/request.h"

class RouterSession {
    pid_t     pid;              /* pid of this process */
    int       readfd;           /* reading from process's stdout */
    int       writefd;          /* writing to process's stdin */
    uint32_t  session_id;       /* session Id */
    time_t    last_access;      /* last access time */
    string_t  name;             /* name of session */
    std::vector<char*> args;    /* arguments for this session. First item unused, last item null */
    Request   answer;           /* Buffer for answer data */
    bool      used;             /* true iff this session was used */
 public:
    RouterSession(string_t name, uint32_t id);
    ~RouterSession();

    int checkConflict(const RouterSession& other, bool thisWinsTie) const;
    int checkConflict(const char* query, bool thisWinsTie) const;

    bool start(const char* server);
    void stop();
    bool restart(const char* server);
    bool isActive() const;

    void executeRequest(Request& req, int fd);

    uint32_t getSessionId() const;
    string_t getName() const;
    time_t   getLastAccessTime() const;
    pid_t    getPid() const;
    bool     wasUsed() const;
};

inline bool
RouterSession::isActive() const
{
    return pid != 0;
}

inline uint32_t
RouterSession::getSessionId() const
{
    return session_id;
}

inline string_t
RouterSession::getName() const
{
    return name;
}

inline time_t
RouterSession::getLastAccessTime() const
{
    return last_access;
}

inline pid_t
RouterSession::getPid() const
{
    return pid;
}

inline bool
RouterSession::wasUsed() const
{
    return used;
}

void writeStr(int fd, const char* str);
void writeStr(int fd, string_t str);

#endif
